@@ -71,6 +71,74 @@ hook_file_time(const gchar *filename)
    return mtime > ctime ? mtime : ctime;
 }
 
+/*
+  Does the command we're offering to run actually exist?
+  Command may be quoted, and anything after a space is presumed to be an
+  argument for the command.
+  Command may be present in PATH.
+ */
+static gboolean
+hook_command_exists(const char *cmd)
+{
+   gboolean result = FALSE;
+   gchar *unquoted = NULL;
+   gchar **cargv = NULL;
+   GError *error = NULL;
+
+   if(!cmd) goto out;
+
+   unquoted = g_shell_unquote(cmd, &error);
+   if(!unquoted) {
+      g_debug_hooks("g_shell_unquote: %s", error->message);
+      goto out;
+   }
+
+   gint cargc = 0;
+   cargv = NULL;
+   if(!g_shell_parse_argv(unquoted, &cargc, &cargv, &error)) {
+      g_debug_hooks("g_shell_parse_argv: %s", error->message);
+      goto out;
+   }
+
+   if(cargc < 1) {
+      g_debug_hooks("g_shell_parse_argv: unexpected output");
+      goto out;
+   }
+
+   if(cargv[0][0] == '/' &&
+	 g_file_test(cargv[0], G_FILE_TEST_EXISTS|G_FILE_TEST_IS_EXECUTABLE)) {
+      result = TRUE;
+      g_debug_hooks("command exists");
+      goto out;
+   }
+
+   g_debug_hooks("'%s' is not a valid absolute path", cargv[0]);
+
+   gchar **pathdirs = g_strsplit(getenv("PATH"), ":", 100);
+   if(!pathdirs) goto out;
+
+   for(int i = 0; pathdirs[i] != NULL; i++) {
+      gchar *pathdir = pathdirs[i];
+      char *fname = g_strdup_printf("%s/%s", pathdir, cargv[0]);
+      if(g_file_test(fname, G_FILE_TEST_EXISTS|G_FILE_TEST_IS_EXECUTABLE)) {
+	 g_debug_hooks("command exists at '%s'", fname);
+	 g_free(fname);
+	 result = TRUE;
+	 goto out;
+      }
+      g_free(fname);
+   }
+
+   g_debug_hooks("failed to find command in PATH");
+
+out:
+   if(error) g_error_free(error);
+   if(cargv) g_strfreev(cargv);
+   if(pathdirs) g_strfreev(pathdirs);
+   if(unquoted) g_free(unquoted);
+   return result;
+}
+
 static gboolean
 hook_file_md5(const gchar *filename, guint8 *md5)
 {
@@ -641,6 +709,15 @@ is_hook_relevant(const gchar *hook_file)
       res = (exitc == 0);
    }
 
+   // check that the Command we are offering exists
+   if(res) {
+      b = rfc822_header_lookup(rfc822, "Command");
+      if(b != NULL) {
+	 g_debug_hooks("found Command: '%s'", b);
+	 res = hook_command_exists(b);
+      }
+   }
+
    g_free(filename);
    rfc822_header_free_all(rfc822);
    return res;
